*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

/*******************************************************************/
/* Date: April 2002                                                */
/* Modified:  April 2002                                           */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                            MACROE_V20.SAS                                 ***/
/***                                                                           ***/
/*** This program calculates variance estimates using the bootstrap weights    ***/
/*** for different types of estimators.  Using SAS Macros, this program can    ***/
/*** calculate variance estimates for totals, ratios and differences between   ***/
/*** ratios.  It can also calculate variance estimates for the parameters      ***/
/*** of a linear regression or logistic regression. This program can also be   ***/
/*** customized for other types of analyses.                                   ***/
/***                                                                           ***/
/*** This program contains the macros that are necessary to use the 	       ***/
/*** BOOTVARE_V20.SAS program.                                                 ***/
/***                                                                           ***/
/*** This program is automatically called by BOOTVARE_V20.SAS and NO MODIFICA- ***/
/*** TIONS SHOULD BE MADE BY THE USER (except for specific cases mentioned in  ***/
/*** BOOTVARE_V20.SAS)                                                         ***/
/***                                                                           ***/
/*********************************************************************************/

options ps=48 ls=120 nonotes;


/*********************************************************************************/
/*** Section 1: Declaration of the Macro Variables                             ***/
/*********************************************************************************/

data time;
  format start datetime16.;
  start=datetime();
  output;
run;

* VARIABLE SPECIFIC TO EACH VERSION OF SAS :     ;

data _NULL_;
if &sysver >= 8 then call symput ('inter' ,'intercept');
else call symput ('inter' ,'intercep');
run;

* VARIABLES TO IDENTIFY RESPONDENTS AND FOR THE INDEX (NEXT SECTION):   ;


data _NULL_;
set &Mfile;
          if realukey ne '' then do; 
                                     call symput ('ident' ,'realukey personid');
                                     call symput ('indx','(id=(realukey personid))'); 
                               end;
          else if uniqueid ne '' then do;   
                                     call symput ('ident' ,'uniqueid');
                                     call symput ('indx','(uniqueid)');  
                                     end;
          else if sampleid ne '' then do;   
                                      call symput ('ident' ,'sampleid personid');
                                      call symput ('indx','(id=(sampleid personid))'); 
                                      end;
          else do;
	             put " ERROR: The required identification variables for the respondents were not found ...";
	             put " ";
	             put " For the NPHS main components, these variables are REALUKEY and PERSONID ";
	             put " For the NPHS institution component, the variable is: UNIQUEID";
	             put " For CCHS, these variables are : SAMPLEID and PERSONID";
	             put " ";
	             put " Please verify that these variables are present in the analysis file ";
	             put " (use the RENAME function if the variable names do not correspond )";
	            end;
run;

OPTIONS notes;


***************************************************************************;
*  SECTION 2: READING IN THE MAIN FILE AND MERGING TO THE WEIGHTS         *;
***************************************************************************;

data Mfile (index=&indx);
   set &Mfile ;
run;

/* Read the bootstrap weights */
/* FWGT is the same weight as on the analysis file */

data bsamp (index=&indx);
  set &bsamp;
  keep fwgt &ident bsw1-bsw&b;
run;

/* Merging the main file and the bootstrap weights */

     data bs_data ;
       merge Mfile (in=in1) bsamp (in=in2);
       by &ident;
       if in1;
     run;

/*  Verification if breakdown variable(s) */ 

%let by=;
%let cla_tmp="&classes";

data _NULL_;
if substr(&cla_tmp,1,1)='.' then do;
                   call symput ('by' ,'*');
                   call symput ('classes' ,'');
				   end;
run;

&by  proc sort data=bs_data;
&by     by &classes;
&by  run;


proc datasets library=work;
    delete Mfile bsamp version tmp;
run;

* RESULTS FILES :     ;

data alltots diffrat bs_reg bs_reglg;
    set _NULL_;
run;

%let result= alltots diffrat bs_reg bs_reglg;


/*********************************************************************************/
/*** Section 3: Declaration of the macros                                      ***/
/*********************************************************************************/

%let printtot=0;
%let printdif=0;
%let printreg=0;
%let printlog=0;

%global dep1 dep2;

*************************************;

%macro total(var);

*************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var;
&by  by &classes;
  output out=ytot
  sum=yhat ybs1-ybs&b;
run;

data est;
  set ytot;
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  bs_var=((&b-1)*(var(of ybs1-ybs&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var";
  var2="none";
  type="Total";
  drop ybs1-ybs&b _type_ _freq_;
run;

data alltots;
   set alltots est;
run;

%let printtot=1;

proc datasets library=work;
   delete ytot est;
run;

%mend total;


*******************************************;

%macro ratio(var1,var2,);

*******************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var1;
&by  by &classes;
  output out=ytot
  sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var2;
&by  by &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

data est;
  merge ytot xtot;
  array ybs{&b};
  array xbs{&b};
  array rbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  yhat=((yhat/xhat)*100);
  do i=1 to &b;
    rbs{i}=((ybs{i}/xbs{i})*100);
  end;
  bs_var=((&b-1)*(var(of rbs1-rbs&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  type="Ratio";
  drop ybs1-ybs&b xbs1-xbs&b rbs1-rbs&b xhat i _type_ _freq_;
run;

data alltots;
    set alltots est;
run;

%let printtot=1;

proc datasets library=work;
    delete ytot xtot est;
run;

%mend ratio;


********************************************************;

%macro diff_rat(var1,var2,var3,var4);

********************************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var1;
&by  by &classes;
  output out=ytot
  sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var2;
&by  by &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var3;
&by  by &classes;
  output out=yytot
  sum=yyhat yybs1-yybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var4;
&by  by &classes;
  output out=xxtot
  sum=xxhat xxbs1-xxbs&b;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&b};
  array xbs{&b};
  array yybs{&b};
  array xxbs{&b};
  array drbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length var3 $ 8;
  length var4 $ 8;
  length type $ 10;
  yhat=(((yhat/xhat)-(yyhat/xxhat))*100);
  do i=1 to &b;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i}))*100);
  end;
  bs_var=(((&b-1)*(var(of drbs1-drbs&b)))/&b);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/yhat)*100,.01));
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  var3="&var3";
  var4="&var4";
  type="Diff_Ratio";
  drop ybs1-ybs&b xbs1-xbs&b yybs1-yybs&b xxbs1-xxbs&b drbs1-drbs&b
       xhat xxhat yyhat i _type_ _freq_;
run;

data diffrat;
    set diffrat est;
run;

%let printdif=1;

proc datasets library=work;
    delete ytot xtot yytot xxtot est;
run;

%mend diff_rat;


*********************************************;

%macro regress(yvar,xvar);

*********************************************;

proc reg data=bs_data outest=orig(keep=&classes &inter &xvar) noprint;
  model &yvar=&xvar;
  weight fwgt;
&by  by &classes;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=bhat name=beta;
  var &inter &xvar;
&by  by &classes;
run;

data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do;	
                          k=&k;
                          j=1+((&k-1)*&L);
                          kL=&k*&L;
                       end;
                  else do; 	
                          k=10; 
                          j=&j_dep;
                          kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;

data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

     %put Regression &i completed;

   %let j_dep=%eval(&kl+1);

   proc reg data=poids outest=betas(keep=&classes &inter &xvar) noprint;
     model &yvar=&xvar;
     weight bsw&i;
&by  by &classes;
   run;

   proc transpose data=betas out=betat prefix=best name=beta;
     var &inter &xvar;
&by  by &classes;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
	if &i=1 then test=1;
	else test=test+1;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by &classes;
     run;

   %end;
  %end;
%end;

OPTIONS notes;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  cil95=bhat1-1.96*bs_sd;
  ciu95=bhat1+1.96*bs_sd;
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reg;
    set bs_reg est;
run;

%let printreg=1;
%let dep1=&yvar;

proc datasets library=work;
    delete betas betat bsbeta origest;
run;


%mend regress;

*********************************************;

%macro logreg(yvar,xvar);

*********************************************;

proc logistic data=bs_data outest=orig(keep=&classes &inter &xvar) descending noprint;
  model &yvar=&xvar;
  &by by &classes;
  weight fwgt;
run;

proc transpose data=orig out=origest prefix=bhat name=beta;
  var &inter &xvar;
  &by by &classes;
run;


data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do;	
                          k=&k;
                          j=1+((&k-1)*&L);
                          kL=&k*&L;
                       end;
                  else do; 	
                          k=10; 
                          j=&j_dep;
                          kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;


data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

     %put Logistic regression &i completed;

     %let j_dep=%eval(&kl+1);

    proc logistic data=poids outest=betas (keep=&classes &inter &xvar) noprint descending;
      model &yvar=&xvar;
      &by by &classes;
      weight bsw&i;
    run;



    proc transpose data=betas out=betat prefix=best name=beta;
      var &inter &xvar;
      &by by &classes;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by &classes;
      run;

    %end;
  %end;
%end;

OPTIONS notes;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  wald=(bhat1/bs_sd)*(bhat1/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=bhat1-1.96*bs_sd;
  hi95=bhat1+1.96*bs_sd;
  odds=exp(bhat1);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reglg; 
    set bs_reglg est;
run;

%let printlog=1;
%let dep2=&yvar;

proc datasets library=work;
    delete betas betat bsbeta origest;
run;


%mend logreg;



****************;

%macro prnttot;

****************;
%if &printtot=1 %then %do;

&by   proc sort data=alltots;
&by     by &classes;
&by   run;

/**************************************************/
/* Prints the results of the total ratio macros   */
/**************************************************/

proc print data=alltots;
    	 title "Variance estimation using &B bootstraps for ";
    	 title2 "Totals and Ratios ";
	 title3 " ";
	 var &classes type var1 var2 yhat bs_sd bs_cv cil95 ciu95;
  	 format yhat bs_sd cil95 ciu95 11.2;
run;

%end;

        /*****************************************************************/
        /*** Where:                                                    ***/
        /*** type         : estimate type (total ratio)                ***/
        /*** var1 et var2 : variables used to calculate the estimates. ***/
        /***                For a total, var2 = blank                  ***/
        /*** yhat         : parameter estimate (in % for ratio)        ***/
        /*** bs_sd        : standard deviation                         ***/
        /*** bs_cv        : coefficient of variation                   ***/
        /*** cil95        : lower bound of the 95% confidence interval ***/
        /*** ciu95        : upper bound of the 95% confidence interval ***/
        /*****************************************************************/


%mend prnttot;

****************;

%macro prntdiff;

****************;
%if &printdif=1 %then %do;

&by   proc sort data=diffrat;
&by     by &classes;
&by   run;



/**********************************************/
/* Prints the results of the diff_rat macro   */
/**********************************************/

proc print data=diffrat;
  	 title "Variance estimation using &B bootstraps for ";
	 title2 "Differences between Ratios ";
	 title3 "";
         var &classes type var1 var2 var3 var4 yhat bs_sd bs_cv cil95 ciu95;
         format yhat bs_sd cil95 ciu95 11.2;
   run;
%end;

        /*****************************************************************/
        /*** Where:                                                    ***/
        /*** type         : estimate type (ratio difference)           ***/
        /*** var1, var2,                                               ***/
        /*** var3 and var4: variables used to calculate the estimates. ***/
        /*** yhat         : estimate (difference in % )                ***/
        /*** bs_sd        : standard deviation                         ***/
        /*** bs_cv        : coefficient of variation                   ***/
        /*** cil95        : lower bound of the 95% confidence interval ***/
        /*** ciu95        : upper bound of the 95% confidence interval ***/
        /*****************************************************************/


%mend prntdiff;

****************;

%macro prntreg;

****************;
%if &printreg=1 %then %do;


/******************************************************/
/* Prints the results of the regress macro            */
/******************************************************/

   proc print data=bs_reg;
     title "Variance estimation using &B bootstraps for a ";
     title2 "Regression ";
     title3 "Dependent variable: &dep1";
     var &classes beta bhat bs_var bs_sd bs_cv cil95 ciu95;
   run;
%end;

        /**************************************************************************/
        /***  Where:                                                            ***/
        /***   beta       : parameter to estimate                               ***/
        /***   bhat       : parameter estimate                                  ***/
        /***   bsvar      : variance of the parameter estimate                  ***/
        /***   bs_sd      : standard deviation of the parameter estimate        ***/
        /***   bs_cv      : coefficient of variation for the parameter estimate ***/
        /***   cil95      : lower bound of the 95% confidence interval          ***/
        /***   ciu95      : upper bound of the 95% confidence interval          ***/
        /**************************************************************************/


%mend prntreg;

****************;

%macro prntlog;

****************;
%if &printlog=1 %then %do;

/********************************************/
/*Prints the results of the logreg macro    */
/********************************************/

   proc print data=bs_reglg;
     title "Variance estimation using &B bootstraps for ";
     title2 "Logistic regressions";
     title3 "Dependent variable: &dep2";
     var  &classes beta bhat odds wald pvalue bs_var bs_sd bs_cv cil95 ciu95;
	 run;
%end;

        /**************************************************************************/
        /***  Where:                                                            ***/
        /***   beta       : parameter to estimate                               ***/
        /***   bhat       : parameter estimate                                  ***/
        /***   odds       : odds ratio                                          ***/
        /***   wald       : Wald's statistic                                    ***/
        /***   pvalue     : p-value of Wald's statistic                         ***/
        /***   bsvar      : variance of the parameter estimate                  ***/
        /***   bs_sd      : standard deviation of the parameter estimate        ***/
        /***   bs_cv      : coefficient of variation for the parameter estimate ***/
        /***   cil95      : lower bound of the 95% confidence interval          ***/
        /***   ciu95      : upper bound of the 95% confidence interval          ***/
        /**************************************************************************/


%mend prntlog;


****************;

%macro output;

****************;

	%prnttot;
	%prntdiff;
	%prntreg;
	%prntlog;

	data time;
 	 set time;
	  format stop datetime16.;
 	 stop=datetime();
	  output;
	run;

	proc print data=time;
  		title ' Length of time required to run the program ';
	run;

%mend output;



/* End of MACRO.SAS SAS program */
